package com.sencha.gxt.explorer.client.forms;

import java.util.Date;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.i18n.client.DateTimeFormat.PredefinedFormat;
import com.google.gwt.i18n.client.NumberFormat;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.util.DateWrapper;
import com.sencha.gxt.data.shared.LabelProvider;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.data.shared.ModelKeyProvider;
import com.sencha.gxt.data.shared.PropertyAccess;
import com.sencha.gxt.examples.resources.client.TestData;
import com.sencha.gxt.examples.resources.client.model.Stock;
import com.sencha.gxt.explorer.client.model.Example.Detail;
import com.sencha.gxt.widget.core.client.event.ParseErrorEvent;
import com.sencha.gxt.widget.core.client.form.ComboBox;
import com.sencha.gxt.widget.core.client.form.DateField;
import com.sencha.gxt.widget.core.client.form.NumberField;
import com.sencha.gxt.widget.core.client.form.NumberPropertyEditor;
import com.sencha.gxt.widget.core.client.form.NumberPropertyEditor.DoublePropertyEditor;
import com.sencha.gxt.widget.core.client.form.SpinnerField;
import com.sencha.gxt.widget.core.client.form.TextArea;
import com.sencha.gxt.widget.core.client.form.validator.MinDateValidator;
import com.sencha.gxt.widget.core.client.form.validator.MinLengthValidator;
import com.sencha.gxt.widget.core.client.info.Info;

@Detail(name = "Forms Example (UiBinder)", icon = "forms", category = "Forms", files = "FormsUiBinderExample.ui.xml")
public class FormsUiBinderExample implements IsWidget, EntryPoint {

  interface StockProperties extends PropertyAccess<Stock> {
    LabelProvider<Stock> name();

    ModelKeyProvider<Stock> symbol();
  }

  interface MyUiBinder extends UiBinder<Widget, FormsUiBinderExample> {
  }

  private static MyUiBinder uiBinder = GWT.create(MyUiBinder.class);

  StockProperties props = GWT.create(StockProperties.class);
  ListStore<Stock> store = new ListStore<Stock>(props.symbol());

  @UiField(provided = true)
  ComboBox<Stock> combo = new ComboBox<Stock>(store, props.name());

  @UiField
  DateField date;

  @UiField
  TextArea description;

  @UiField(provided = true)
  SpinnerField<Double> spinnerField;

  @UiField(provided = true)
  NumberField<Integer> age;
  
  @UiField(provided = true)
  Date minValue = new DateWrapper().clearTime().addHours(8).asDate();
  
  @UiField(provided = true)
  Date maxValue = new DateWrapper().clearTime().addHours(18).addSeconds(1).asDate();
  
  
  public Widget asWidget() {
    store.addAll(TestData.getStocks());

    spinnerField = new SpinnerField<Double>(new DoublePropertyEditor());
    age = new NumberField<Integer>(new NumberPropertyEditor.IntegerPropertyEditor());

    Widget widget = uiBinder.createAndBindUi(this);

    date.addValidator(new MinDateValidator(new Date()));
    description.addValidator(new MinLengthValidator(10));
    spinnerField.setIncrement(.1d);
    spinnerField.setMinValue(-10d);
    spinnerField.setMaxValue(10d);
    spinnerField.getPropertyEditor().setFormat(NumberFormat.getFormat("00.0"));

    return widget;
  }

  public void onModuleLoad() {
    RootPanel.get().add(asWidget());
  }

  @UiHandler("firstName")
  public void firstNameChanged(ValueChangeEvent<String> event) {
    Info.display("Value Changed", "First name changed to " + event.getValue() == null ? "blank" : event.getValue());
  }

  @UiHandler("age")
  public void onAgeParseError(ParseErrorEvent event) {
    Info.display("Parse Error", event.getErrorValue() + " could not be parsed as a number");
  }

  @UiHandler("date")
  public void onDateParseError(ParseErrorEvent event) {
    Info.display("Parse Error", event.getErrorValue() + " could not be parsed as a date");
  }

  @UiHandler("date")
  public void onDateChanged(ValueChangeEvent<Date> event) {
    String v = event.getValue() == null ? "nothing" : DateTimeFormat.getFormat(PredefinedFormat.DATE_MEDIUM).format(
        event.getValue());
    Info.display("Selected", "You selected " + v);
  }
  
  @UiHandler("time")
  public void onTimeParseError(ParseErrorEvent event) {
    Info.display("Parse Error", event.getErrorValue() + " could not be parsed as a valid time.");
  }
}
